@@ -17,6 +17,7 @@ By convention, all attributes are read-only, although this is not enforced at ru
 """
 from __future__ import annotations
 
+import dataclasses
 import enum
 import inspect
 import os
@@ -27,22 +28,30 @@ import textwrap
 import traceback
 import types
 import warnings
-from abc import ABCMeta, abstractmethod
+from abc import ABCMeta
+from abc import abstractmethod
 from collections.abc import Callable
 from functools import wraps
 from pathlib import Path
-from typing import Any, ClassVar, Generic, TypeVar, Union
+from typing import Any
+from typing import ClassVar
+from typing import Generic
+from typing import TypeVar
+from typing import Union
 
-from pdoc import doc_ast, doc_pyi, extract
-from pdoc.doc_types import (
-    GenericAlias,
-    NonUserDefinedCallables,
-    empty,
-    resolve_annotations,
-    safe_eval_type,
-)
-
-from ._compat import cache, cached_property, formatannotation, get_origin, singledispatchmethod
+from ._compat import cache
+from ._compat import cached_property
+from ._compat import formatannotation
+from ._compat import get_origin
+from ._compat import singledispatchmethod
+from pdoc import doc_ast
+from pdoc import doc_pyi
+from pdoc import extract
+from pdoc.doc_types import empty
+from pdoc.doc_types import GenericAlias
+from pdoc.doc_types import NonUserDefinedCallables
+from pdoc.doc_types import resolve_annotations
+from pdoc.doc_types import safe_eval_type
 
 
 def _include_fullname_in_traceback(f):
@@ -102,6 +111,19 @@ class Doc(Generic[T]):
     in the context of classes, this points to the class an attribute is inherited from.
     """
 
+    kind: ClassVar[str]
+    """
+    The type of the doc object, either `"module"`, `"class"`, `"function"`, or `"variable"`.
+    """
+
+    @property
+    def type(self) -> str:  # pragma: no cover
+        warnings.warn(
+            "pdoc.doc.Doc.type is deprecated. Use pdoc.doc.Doc.kind instead.",
+            DeprecationWarning,
+        )
+        return self.kind
+
     def __init__(
         self, modulename: str, qualname: str, obj: T, taken_from: tuple[str, str]
     ):
@@ -177,20 +199,6 @@ class Doc(Generic[T]):
         """
         return (self.modulename, self.qualname) != self.taken_from
 
-    @classmethod
-    @property
-    def type(cls) -> str:
-        """
-        The type of the doc object, either `"module"`, `"class"`, `"function"`, or `"variable"`.
-        """
-        return cls.__name__.lower()
-
-    if sys.version_info < (3, 9):  # pragma: no cover
-        # no @classmethod @property in 3.8
-        @property
-        def type(self) -> str:  # noqa
-            return self.__class__.__name__.lower()
-
     def __lt__(self, other):
         assert isinstance(other, Doc)
         return self.fullname.replace("__init__", "").__lt__(
@@ -374,6 +382,8 @@ class Module(Namespace[types.ModuleType]):
         """
         super().__init__(module.__name__, "", module, (module.__name__, ""))
 
+    kind = "module"
+
     @classmethod
     @cache
     def from_name(cls, name: str) -> Module:
@@ -406,7 +416,7 @@ class Module(Namespace[types.ModuleType]):
         for k, v in _safe_getattr(self.obj, "__annotations__", {}).items():
             annotations[k] = v
 
-        return resolve_annotations(annotations, self.obj, self.fullname)
+        return resolve_annotations(annotations, self.obj, None, self.fullname)
 
     def _taken_from(self, member_name: str, obj: Any) -> tuple[str, str]:
         if obj is empty:
@@ -548,6 +558,8 @@ class Class(Namespace[type]):
     Representation of a class's documentation.
     """
 
+    kind = "class"
+
     @cache
     @_include_fullname_in_traceback
     def __repr__(self):
@@ -559,8 +571,16 @@ class Class(Namespace[type]):
         if doc == dict.__doc__:
             # Don't display default docstring for dict subclasses (primarily TypedDict).
             return ""
-        else:
-            return doc
+        is_dataclass_with_default_docstring = (
+            dataclasses.is_dataclass(self.obj)
+            # from https://github.com/python/cpython/blob/3.10/Lib/dataclasses.py
+            and doc
+            == self.obj.__name__
+            + str(inspect.signature(self.obj)).replace(" -> None", "")
+        )
+        if is_dataclass_with_default_docstring:
+            return ""
+        return doc
 
     @cached_property
     def _var_docstrings(self) -> dict[str, str]:
@@ -594,8 +614,9 @@ class Class(Namespace[type]):
                 if attr not in annotations
                 or annotations[attr][0] is not unresolved_annotation
             }
+            localns = _safe_getattr(cls, "__dict__", None)
             for attr, t in resolve_annotations(
-                new_annotations, inspect.getmodule(cls), cls_fullname
+                new_annotations, inspect.getmodule(cls), localns, cls_fullname
             ).items():
                 annotations[attr] = (new_annotations[attr], t)
 
@@ -644,6 +665,8 @@ class Class(Namespace[type]):
                 unsorted.setdefault(name, obj)
         for name in self._var_docstrings:
             unsorted.setdefault(name, empty)
+        for name in self._var_annotations:
+            unsorted.setdefault(name, empty)
 
         init_has_no_doc = unsorted.get("__init__", object.__init__).__doc__ in (
             None,
@@ -794,6 +817,8 @@ class Function(Doc[types.FunctionType]):
     supports `@classmethod`s or `@staticmethod`s.
     """
 
+    kind = "function"
+
     wrapped: WrappedFunction
     """The original wrapped function (e.g., `staticmethod(func)`)"""
 
@@ -915,17 +940,23 @@ class Function(Doc[types.FunctionType]):
             )
         mod = inspect.getmodule(self.obj)
         globalns = _safe_getattr(mod, "__dict__", {})
+        localns = globalns
+        for parent_cls_name in self.qualname.split(".")[:-1]:
+            parent_cls = localns.get(parent_cls_name, object)
+            localns = _safe_getattr(parent_cls, "__dict__", None)
+            if localns is None:
+                break  # pragma: no cover
 
         if self.name == "__init__":
             sig = sig.replace(return_annotation=empty)
         else:
             sig = sig.replace(
                 return_annotation=safe_eval_type(
-                    sig.return_annotation, globalns, mod, self.fullname
+                    sig.return_annotation, globalns, localns, mod, self.fullname
                 )
             )
         for p in sig.parameters.values():
-            p._annotation = safe_eval_type(p.annotation, globalns, mod, self.fullname)  # type: ignore
+            p._annotation = safe_eval_type(p.annotation, globalns, localns, mod, self.fullname)  # type: ignore
         return sig
 
     @cached_property
@@ -944,6 +975,8 @@ class Variable(Doc[None]):
     Representation of a variable's documentation. This includes module, class and instance variables.
     """
 
+    kind = "variable"
+
     default_value: Any | empty  # technically Any includes empty, but this conveys intent.
     """
     The variable's default value.
